/*******************************************************************************
 * Copyright (c) 2013, SWITCH
 ******************************************************************************/
package ch.SWITCH.uniapp;

import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

public class AAILoginActivity extends BaseLoginActivity {
	protected static final String LOG_TAG = "AAILoginActivity";

	/**
	 * Starts a webbrowser Intent for an AAI browser login.
	 */
	private void launchAAILoginBrowser() {
		Intent intent = new Intent(Intent.ACTION_VIEW);
		Uri uri = Util.getApiUriBuilder(this)
				.appendPath("signin")
				.appendPath(getString(R.string.uniapp_id))
				.appendQueryParameter("device", Util.getDeviceName())
				.appendQueryParameter("client_id", getString(R.string.uniapp_client_id))
				.appendQueryParameter("redirect_uri", "uniapp://" + getString(R.string.uniapp_id))
				.appendQueryParameter("response_type", "token")
				.appendQueryParameter("scope", "all")
				.build();
		intent.setData(uri);
		Log.d(LOG_TAG, "Opening browser: " + uri.toString());
		startActivity(intent);
	}

	/**
	 * Checks the Activity starting Intent for an Access Token to set
	 * 
	 * @throws IllegalArgumentException
	 *             if Intent data exists, but doesn't contain a valid access
	 *             token and expiry time
	 */
	protected void consumeIntentData() throws IllegalArgumentException {
		Uri data = getIntent().getData();
		if (data != null) {
			String token = data.getQueryParameter("access_token");
			if (token == null)
				throw new IllegalArgumentException("access_token missing in intent url");
			String expires_s = data.getQueryParameter("expires");
			if (expires_s == null)
				throw new IllegalArgumentException("expires missing in intent url");
			int expires = Integer.parseInt(expires_s);
			Log.d(LOG_TAG, "Assigning new token '" + token + "'");
			setAccessToken(this, token, expires);
		}
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.screen_splash);

		// check if the activity was started by intent
		try {
			consumeIntentData();
		} catch (IllegalArgumentException e) {
			showErrorDialog("The link into your App was invalid.", false);
			Log.w(LOG_TAG, "Intent URL was invalid", e);
		}

		// checking for access token expiry
		String accessToken;
		try {
			accessToken = getAccessToken(this);
		} catch (TokenExpiredException e) {
			accessToken = null;
			Toast.makeText(this, "Your login expired.", Toast.LENGTH_LONG)
			.show();
		}

		// if no token exists, show the login screen
		if (accessToken == null) {
			Log.d(LOG_TAG, "No access token found, displaying login screen");
			displayLoginScreen();

			// otherwise query AAI attributes and go to the home screen
		} else {
			// queryAndLaunchHome(accessToken);
			new QueryAndLauchHomeTask(new AAIAttributeStorage(this), accessToken).execute();
		}

	}

	/**
	 * Displays the login screen
	 */
	private void displayLoginScreen() {
		setContentView(R.layout.activity_aailogin);
		Button vbLogin = (Button) findViewById(R.id.button_login);
		vbLogin.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				launchAAILoginBrowser();
				finish();

			}
		});

		Button vbAltLogin = (Button) findViewById(R.id.button_altlogin);
		vbAltLogin.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				Intent intent = new Intent(getApplicationContext(), AlternateLoginActivity.class);
				startActivity(intent);
			}
		});

		((Button) findViewById(R.id.button_altlogin_i)).setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				Builder builder = new AlertDialog.Builder(AAILoginActivity.this);
				builder.setTitle("Info");
				builder.setMessage("Use this if your Identity Provider uses an exotic login method like certificate login, that cannot be completed on a phone. This will allow you to log in useing your computer.");
				builder.setNeutralButton("Ok", new OnClickListener() {

					@Override
					public void onClick(DialogInterface dialog, int which) {
						dialog.dismiss();
					}
				});
				builder.create().show();

			}
		});

	}

}
