/*******************************************************************************
 * Copyright (c) 2013, SWITCH
 ******************************************************************************/
package ch.SWITCH.uniapp;

import java.io.BufferedReader;
import java.io.InputStreamReader;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;

import android.os.AsyncTask;
import android.util.Log;

import com.google.gson.Gson;
import com.google.gson.JsonParseException;

/**
 * @author latzer
 *
 * Base extension of AsyncTask to acces the Mobile Proxy API
 */
public abstract class ApiCallTask extends AsyncTask<String, Long, JSend> {
	private static final String LOG_TAG = "ApiCallTask";

	/**
	 * Issues a GET request to the given URL
	 * 
	 * @param requestUrl
	 * @return the HTTP response as text
	 */
	protected String apiCall(String requestUrl){
		HttpClient client = new DefaultHttpClient();
		HttpGet request = new HttpGet(requestUrl);
		try{
			Log.d(LOG_TAG, "Requesting " + requestUrl);
			HttpResponse response = client.execute(request);
			Log.v(LOG_TAG, "HTTP Status " +response.getStatusLine().getStatusCode());

			// Get the response
			BufferedReader rd = new BufferedReader
					(new InputStreamReader(response.getEntity().getContent()));

			StringBuffer bResponse = new StringBuffer();
			String line;
			while ((line = rd.readLine()) != null) {
				bResponse.append(line);
			}
			String responseString = bResponse.toString();
			Log.d(LOG_TAG, "response: " + responseString);

			return responseString;


		} catch ( Throwable e){
			Log.e(LOG_TAG, "apiCall failed", e);
			return null;
		}

	}

	/**
	 * Issues a GET request to the given URL
	 * @param requestUrl
	 * @return the response parsed as JSend object
	 */
	protected JSend apiCallJSend(String requestUrl){
		try{
			String jsonString = apiCall(requestUrl);
			if(jsonString == null){
				JSend error = new JSend();
				error.setStatus(JSend.ERROR);
				error.setMessage("Error in communicating with server");
				return error;
			}
			JSend response = new Gson().fromJson(jsonString, JSend.class);
			if(response.getStatus() == null){
				throw new JsonParseException("Got no valid JSend object as response!");
			}
			return response;
		} catch ( JsonParseException e){
			Log.e(LOG_TAG, "unable to parse response", e);
			JSend error = new JSend();
			error.setStatus(JSend.ERROR);
			error.setMessage("Unable to parse response.");
			return error;
		}

	}

}
