<?php
/* Copyright (c) 2013, SWITCH */

class ClientValidationFailedException extends Exception{};
class ClientModel implements \OAuth2\Storage\ClientInterface {
	private $db;

	public function __construct() {
			require_once 'lib/DB.php';
			$this->db = new DB();
	}

	public function getClient($clientId = null, $clientSecret = null, $redirectUri = null) {
		if(!$clientId){
			throw new Exception("clientId cannot be null");
		}
		if(!$redirectUri){
			throw new Exception("redirectUri");
		}
		if($redirectUri == "none"){
			//this can happen if we do a manual token login
			$res = $this->db->query('
				SELECT 
					oauth_clients.id as client_id, 
					secret as client_secret, 
					name 
				FROM oauth_clients, oauth_client_endpoints
					WHERE client_id = oauth_clients.id
					AND client_id = ?', 
				array($clientId)
			);
		} else {
			$res = $this->db->query('
				SELECT 
					oauth_clients.id as client_id, 
					secret as client_secret, 
					name 
				FROM oauth_clients, oauth_client_endpoints
					WHERE client_id = oauth_clients.id
					AND client_id = ?
					AND redirect_uri = ?', 
				array($clientId, $redirectUri)
			);
		}

		$res->setFetchMode(PDO::FETCH_ASSOC);
		return $res->fetch(); //either the client, or false if it wasn't found
	}

	/**
	 * Returns the id of the client with given name
	 * @param String name
	 * @return the id of the given client, or false if no such client exists
	 *
	 */
	public function getClientId($name){
		$res = $this->db->query('
			SELECT
				id
			FROM oauth_clients
			WHERE name = ?',
		array($name));
		$result = $res->fetch();
		if($result){
			return $result->id;
		}
		return false;
	}

	/**
	 * Verifies that the request contains an authorization Header with
	 * credentials that belong to a configured client
	 */
	public function validateClientAuth(){
		$id = @$_SERVER['PHP_AUTH_USER'];
		$secret = @$_SERVER['PHP_AUTH_PW'];
		if(!$id || !$secret){
			throw new ClientValidationFailedException("Cannot process basic authorization header");
		}
		$res = $this->db->query('
			SELECT
				id
			FROM oauth_clients
				WHERE id = ?
				AND secret = ?',
			array($id, $secret)
		);
		if($res->fetch()){
			return true;
		}
		return false;
	}

}
