<?php
/* Copyright (c) 2013, SWITCH */

use OAuth2\Request;
use OAuth2\AuthServer;
use OAuth2\Exception;
use OAuth2\Util\SecureKey;
use OAuth2\Storage\SessionInterface;
use OAuth2\Storage\ClientInterface;
use OAuth2\Storage\ScopeInterface;
use OAuth2\Grant\GrantTypeInterface;

/*
 * Implicit oauth grant - an access token is issued directly, without using an authorization code
 */
class Implicit implements GrantTypeInterface{
	
	public function __construct(AuthServer $auth){
	
	}
	
	public function getIdentifier(){
		return "implicit";
	}

	public function getResponseType(){
		return "token";
	}

	public function completeFlow($inputParams = null){
		// Get the required params
		$authParams = AuthServer::getParam(array('client_id', 'redirect_uri', 'code', 'user_id', 'persistent-id', 'device'), 'post', $inputParams);

		if (is_null($authParams['client_id'])) {
			throw new Exception\ClientException(sprintf(AuthServer::getExceptionMessage('invalid_request'), 'client_id'), 0);
		}

		if (is_null($authParams['redirect_uri'])) {
			throw new Exception\ClientException(sprintf(AuthServer::getExceptionMessage('invalid_request'), 'redirect_uri'), 0);
		}

		if (is_null($authParams['user_id'])) {
			throw new Exception\ClientException(sprintf(AuthServer::getExceptionMessage('invalid_request'), 'user_id'), 0);
		}

		// Validate client ID and redirect URI
		$clientDetails = AuthServer::getStorage('client')->getClient($authParams['client_id'], null, $authParams['redirect_uri']);

		if ($clientDetails === false) {
			throw new Exception\ClientException(AuthServer::getExceptionMessage('invalid_client'), 8);
		}

		$authParams['client_details'] = $clientDetails;

		// A session ID was returned so update it with an access token,
		//	remove the authorisation code, change the stage to 'granted'

		$accessToken = SecureKey::make();
		$refreshToken = (AuthServer::hasGrantType('refresh_token')) ? SecureKey::make() : null;

		$accessTokenExpires = time() + AuthServer::getExpiresIn();
		$accessTokenExpiresIn = AuthServer::getExpiresIn();

		// Create a new session
		$sessionID = AuthServer::getStorage('session')->createSession(
			$authParams['client_id'],
			$authParams['redirect_uri'],
			'client',
			$authParams['user_id'],
			null,
			$accessToken,
			$refreshToken,
			$accessTokenExpires,
			'granted',
			$authParams['persistent-id'],
			$authParams['device']
		);

		$response = array(
			'access_token'	=>	$accessToken,
			'token_type'	=>	'bearer',
			'expires'		=>	$accessTokenExpires,
			'expires_in'	=>	$accessTokenExpiresIn
		);

		if (AuthServer::hasGrantType('refresh_token')) {
			$response['refresh_token'] = $refreshToken;
		}

		return $response;
	}
}
